/*
Project name      : LCD 2x16 Bargraph - Progress bar v1.0
Author            : Petar Timotijevic, ptimotijevic@gmail.com
Development board : EasyPIC v7
Microcontroller   : PIC18F45K22 32MHz, 8MHz crystal oscillator, PLL Enabled
Required Libraries: ADC, LCD, Conversions
Description       : Read ADC voltage, show ADC steps in first row, bargraph in second row
                    ADC input pin is RA0	    
*/


// LCD pins definition
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;

unsigned int adc_rd;
unsigned long int BarSegmentNum;
char adcprint, i;


void main() 
{
     ANSELA.F0 = 1;            // Configure RA0 pin as analog
     TRISA.F0  = 1;            // Configure RA0 pin as input

     Lcd_Init();               // Initialize LCD
     Lcd_Cmd(_LCD_CLEAR);      // Clear LCD
     Lcd_Cmd(_LCD_CURSOR_OFF); // Turn cursor off


     while(1)
         {
         adc_rd = ADC_Read(0);         // Get ADC value from 1st channel
         wordtostr(adc_rd, adcprint);  // Conversion to string
         Lcd_out(1,5,adcprint);        // Prints ADC steps in LCD first row

         BarSegmentNum = adc_rd/(1023/16); // Calculate numbers of bar graph segments

         Lcd_out(2,1,"                ");  // Clears LCD second row

         for (i=0;i<BarSegmentNum;i++)     // Loop to print each bar segment in second row
               {
               Lcd_out(2,i+1,">");         // Prints bar graph segment symbol
               }

         Delay_ms(500);                    // Stabilize LCD display
         }

}