/*
Project name      : LCD 2x16 Bargraph - Progress bar v1.2
Author            : Petar Timotijevic, ptimotijevic@gmail.com
Development board : EasyPIC v7
Microcontroller   : PIC18F45K22 32MHz, 8MHz crystal oscillator, PLL Enabled
Required Libraries: ADC, LCD, Conversions
Description       : First row shows ADC steps as percentage from 0 to 100%
                    Second row shows bargraph segments from 0 to 16 (for 2x16 LCD)
                    ADC input pin is RA0
                    Formula to calc percentage of ADC steps: ADC_steps/(Max_steps/100)
*/


// LCD pins definition
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;

unsigned int adc_rd, diff, adc_perc;
unsigned long int BarSegmentNum;
char adcprint, i;

const char character[] = {31,31,31,31,31,31,31,31};

void CustomChar(char pos_row, char pos_char) 
     {
     char ip;
     Lcd_Cmd(64);
     for (ip = 0; ip<=7; ip++) Lcd_Chr_CP(character[ip]);
     Lcd_Cmd(_LCD_RETURN_HOME);
     Lcd_Chr(pos_row, pos_char, 0);
     }

void main()
{
     ANSELA.F0 = 1;            // Configure RA0 pin as analog
     TRISA.F0  = 1;            // Configure RA0 pin as input

     Lcd_Init();               // Initialize LCD
     Lcd_Cmd(_LCD_CLEAR);      // Clear LCD
     Lcd_Cmd(_LCD_CURSOR_OFF); // Turn cursor off


     while(1)
         {
         adc_rd = ADC_Read(0);         // Get ADC value from 1st channel
         adc_perc = adc_rd/(1023/100); // Formula for percentage is ADC_Steps/(Max_Steps/100)
         if (adc_perc > 0) adc_perc = adc_perc - 2;
         inttostr(adc_perc, adcprint); // Conversion to string
         Lcd_out(1,4,adcprint);        // Prints ADC steps in LCD first row
         Lcd_chr(1,10,'%');

         Lcd_out(1,1,"0%");
         Lcd_out(1,13,"100%");

         if (diff != adc_rd)
              {
              BarSegmentNum = adc_rd/(1023/16); // Calculate numbers of bar graph segments

              Lcd_out(2,1,"                ");  // Clears LCD second row

              for (i=0;i<BarSegmentNum;i++)     // Loop to print each bar segment in second row
                    {
                    CustomChar(2,i+1);          // Print custom character for bargraph segment
                    }
              }
         Delay_ms(1500);                   // Stabilize LCD display

         diff = adc_rd;
         }
}