/*
Project name      : LCD 2x16 Bargraph - Progress bar v1.3
Author            : Petar Timotijevic, ptimotijevic@gmail.com
Development board : EasyPIC v7
Microcontroller   : PIC18F45K22 32MHz, 8MHz crystal oscillator, PLL Enabled
Required Libraries: ADC, LCD, Conversions
Description       : First row shows ADC steps as numbers from 1 to 5
                    Second row shows bar graph custom characters segments from 1 to 5
                    ADC input pin is RA0
*/


// LCD pins definition
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;

unsigned int adc_rd, diff;
unsigned long int BarSegmentNum;
char adcprint, i, ip;

const char character1[] = {16,16,16,16,16,16,16,16};
const char character2[] = {24,24,24,24,24,24,24,24};
const char character3[] = {28,28,28,28,28,28,28,28};
const char character4[] = {30,30,30,30,30,30,30,30};
const char character5[] = {31,31,31,31,31,31,31,31};


void main()
{
     ANSELA.F0 = 1;            // Configure RA0 pin as analog
     TRISA.F0  = 1;            // Configure RA0 pin as input

     Lcd_Init();               // Initialize LCD
     Lcd_Cmd(_LCD_CLEAR);      // Clear LCD
     Lcd_Cmd(_LCD_CURSOR_OFF); // Turn cursor off


//1 segment symbol
     Lcd_Cmd(64);
     for (ip=0;ip<=7;ip++) Lcd_Chr_Cp(character1[ip]);
//2 segment symbol
     Lcd_Cmd(72);
     for (ip=0;ip<=7;ip++) Lcd_Chr_Cp(character2[ip]);
//3 segment symbol
     Lcd_Cmd(80);
     for (ip=0;ip<=7;ip++) Lcd_Chr_Cp(character3[ip]);
//4 segment symbol
     Lcd_Cmd(88);
     for (ip=0;ip<=7;ip++) Lcd_Chr_Cp(character4[ip]);
//5 segment symbol
     Lcd_Cmd(96);
     for (ip=0;ip<=7;ip++) Lcd_Chr_Cp(character5[ip]);

     ADC_Init();

     while(1)
         {
         adc_rd = ADC_Get_Sample(0);         // Get ADC value from 1st channel

         if (diff != adc_rd)
             {
             Lcd_out(1,1,"       ");
             Lcd_out(2,1,"       ");

             if (adc_rd < 205)
                       {
                       Lcd_out(1,1,"1");
                       Lcd_Chr(2,1, 0);
                       }
             if ((adc_rd > 205) && (adc_rd < 410))
                       {
                       Lcd_out(1,1,"1");
                       Lcd_Chr(2,1, 0);

                       Lcd_out(1,2,"2");
                       Lcd_Chr(2,2, 1);
                       }
             if ((adc_rd > 410) && (adc_rd < 615))
                       {
                       Lcd_out(1,1,"1");
                       Lcd_Chr(2,1, 0);

                       Lcd_out(1,2,"2");
                       Lcd_Chr(2,2, 1);

                       Lcd_out(1,3,"3");
                       Lcd_Chr(2,3, 2);
                       }
             if ((adc_rd > 615) && (adc_rd < 820))
                       {
                       Lcd_out(1,1,"1");
                       Lcd_Chr(2,1, 0);

                       Lcd_out(1,2,"2");
                       Lcd_Chr(2,2, 1);

                       Lcd_out(1,3,"3");
                       Lcd_Chr(2,3, 2);

                       Lcd_out(1,4,"4");
                       Lcd_Chr(2,4, 3);
                       }                       
             if (adc_rd > 820)
                       {
                       Lcd_out(1,1,"1");
                       Lcd_Chr(2,1, 0);

                       Lcd_out(1,2,"2");
                       Lcd_Chr(2,2, 1);

                       Lcd_out(1,3,"3");
                       Lcd_Chr(2,3, 2);

                       Lcd_out(1,4,"4");
                       Lcd_Chr(2,4, 3);

                       Lcd_out(1,5,"5");
                       Lcd_Chr(2,5, 4);
                       }
             }
        Delay_ms(1000);  // Stabilize LCD display
        diff = adc_rd;
        }

}